@@ -35,11 +35,15 @@ static void parse_configuration(char *, cmdline *, GPtrArray *);
 "       ldapvi --delete [OPTION]... DN...          Edit a delete record\n"    \
 "       ldapvi --rename [OPTION]... DN1 DN2        Edit a rename record\n"    \
 "\n"									      \
+"Configuration profiles:\n"						      \
+"  -p, --profile NAME     Section of ~/.ldapvirc or /etc/ldap.conf to use.\n" \
+"\n"									      \
 "Connection options:\n"							      \
 "  -h, --host URL         Server.\n"					      \
 "  -D, --user USER        Search filter or DN: User to bind as.     [1]\n"    \
 "                         Sets --bind simple.\n"                              \
 "  -w, --password SECRET  Password (also valid for SASL).\n"		      \
+"  -y, --password-file FILE  Password file (also valid for SASL).\n"	      \
 "      --bind [simple,sasl]\n"						      \
 "                         Disable or enable SASL.\n"			      \
 "      --bind-dialog [never,auto,always]\n"				      \
@@ -115,7 +119,7 @@ enum ldapvi_option_numbers {
 	OPTION_NOQUESTIONS, OPTION_LDAPSEARCH, OPTION_LDAPMODIFY,
 	OPTION_LDAPDELETE, OPTION_LDAPMODDN, OPTION_LDAPMODRDN, OPTION_ADD,
 	OPTION_CONFIG, OPTION_READ, OPTION_LDAP_CONF, OPTION_BIND,
-	OPTION_BIND_DIALOG
+	OPTION_BIND_DIALOG, OPTION_UNPAGED_HELP
 };
 
 static struct poptOption options[] = {
@@ -131,6 +135,7 @@ static struct poptOption options[] = {
 	{"sasl-authzid",'X', POPT_ARG_STRING, 0, 'X', 0, 0},
 	{"sasl-authcid",'U', POPT_ARG_STRING, 0, 'U', 0, 0},
 	{"password",	'w', POPT_ARG_STRING, 0, 'w', 0, 0},
+	{"password-file",'y',POPT_ARG_STRING, 0, 'y', 0, 0},
 	{"chase",	'C', POPT_ARG_STRING, 0, 'C', 0, 0},
 	{"deref",	'a', POPT_ARG_STRING, 0, 'a', 0, 0},
 	{"sort",	'S', POPT_ARG_STRING, 0, 'S', 0, 0},
@@ -170,14 +175,17 @@ static struct poptOption options[] = {
 	{"ldapdelete",	  0, 0, 0, OPTION_LDAPDELETE, 0, 0},
 	{"ldapmoddn",	  0, 0, 0, OPTION_LDAPMODDN, 0, 0},
 	{"ldapmodrdn",	  0, 0, 0, OPTION_LDAPMODRDN, 0, 0},
+	{"unpaged-help",  0, 0, 0, OPTION_UNPAGED_HELP, 0, 0},
 	{0, 0, 0, 0, 0}
 };
 
 
+static int usage_pagerp = 1;
+
 void
 usage(int fd, int rc)
 {
-	if (fd == -1 && rc == 0 && isatty(1)) {
+	if (usage_pagerp && fd == -1 && rc == 0 && isatty(1)) {
 		int fd;
 		int pid = pipeview(&fd);
 		write(fd, USAGE, strlen(USAGE));
@@ -205,7 +213,7 @@ init_cmdline(cmdline *cmdline)
 	cmdline->managedsait = 0;
 	cmdline->sortkeys = 0;
 	cmdline->starttls = 0;
-	cmdline->tls = LDAP_OPT_X_TLS_TRY;
+	cmdline->tls = LDAP_OPT_X_TLS_HARD;
 	cmdline->deref = LDAP_DEREF_NEVER;
 	cmdline->verbose = 0;
 	cmdline->noquestions = 0;
@@ -231,6 +239,30 @@ init_cmdline(cmdline *cmdline)
         cmdline->bind_options.sasl_secprops = 0;
 }
 
+static void
+read_password_file(bind_options *bind_options, char *filename)
+{
+	struct stat st;
+	int fd;
+	char *data;
+	char *ptr;
+
+	if (stat(filename, &st) == -1) {
+		fprintf(stderr, "Failed to stat password file: %s\n",
+			filename);
+		exit(1);
+	}
+	data = xalloc(st.st_size + 1);
+	if ( (fd = open(filename, O_RDONLY)) == -1) syserr();
+	if (read(fd, data, st.st_size) != st.st_size) syserr();
+	close(fd);
+	data[st.st_size] = 0;
+	if ( (ptr = strchr(data, '\n')))
+	     *ptr = 0;
+
+	bind_options->password = data;
+}
+
 static void
 parse_argument(int c, char *arg, cmdline *result, GPtrArray *ctrls)
 {
@@ -264,6 +296,9 @@ parse_argument(int c, char *arg, cmdline *result, GPtrArray *ctrls)
 	case 'w':
 		result->bind_options.password = arg;
 		break;
+	case 'y':
+		read_password_file(&result->bind_options, arg);
+		break;
 	case 'd':
 		result->discover = 1;
 		break;
@@ -479,6 +514,9 @@ parse_argument(int c, char *arg, cmdline *result, GPtrArray *ctrls)
 	case OPTION_LDAP_CONF:
 		result->profileonlyp = 0;
 		break;
+	case OPTION_UNPAGED_HELP:
+		usage_pagerp = 0;
+		break;
 	case 'p':
 		parse_configuration(arg, result, ctrls);
 		break;
@@ -577,7 +615,7 @@ parse_configuration(char *profile_name, cmdline *result, GPtrArray *ctrls)
 		}
 		if (!p)
 			break;
-		if (strcmp(entry_dn(p), profile_name)) 
+		if (strcmp(entry_dn(p), profile_name))
 			entry_free(p);
 		else if (profile_found)
 			duplicate = 1;
@@ -629,11 +667,21 @@ parse_arguments(int argc, const char **argv, cmdline *result, GPtrArray *ctrls)
 	}
 	parse_configuration(profile, result, ctrls);
 
-	poptResetContext(ctx);
-	while ( (c = poptGetNextOpt(ctx)) > 0) {
-		char *arg = (char *) poptGetOptArg(ctx);
-		if (c != 'p')
+	{
+		int profile_basedns = result->basedns->len;
+		int cleared_bases = 0;
+
+		poptResetContext(ctx);
+		while ( (c = poptGetNextOpt(ctx)) > 0) {
+			char *arg = (char *) poptGetOptArg(ctx);
+			if (c == 'p') continue;
+			if (c == 'b' && !cleared_bases && profile_basedns) {
+				g_ptr_array_remove_range(
+					result->basedns, 0, profile_basedns);
+				cleared_bases = 1;
+			}
 			parse_argument(c, arg, result, ctrls);
+		}
 	}
 	if (c != -1) {
 		fprintf(stderr, "%s: %s\n",
@@ -683,10 +731,10 @@ parse_arguments(int argc, const char **argv, cmdline *result, GPtrArray *ctrls)
 		break;
 	default:
 		abort();
-	}		
+	}
 
 	if (result->profileonlyp)
-		if (setenv("LDAPNOINIT", "thanks", 1)) syserr();
+		if (putenv("LDAPNOINIT=thanks")) syserr();
 
 	/* don't free! */
 /* 	poptFreeContext(ctx); */
