@@ -46,22 +46,32 @@ compare_ptr_arrays(GPtrArray *a, GPtrArray *b,
 		for (; i < a->len; i++) note(g_ptr_array_index(a, i), 0, x);
 }
 
-static void
-note_values(GArray *a, GArray *b, int *changed)
+static int
+ordered_array_equal(GPtrArray *a, GPtrArray *b,
+		  int (*cmp)(const void *, const void *))
 {
-	if (!(a && b)) *changed = 1;
+	int i;
+
+	if (a->len != b->len)
+		return 0;
+
+	for (i = 0; i < a->len; i++) {
+		void *ax = g_ptr_array_index(a, i);
+		void *bx = g_ptr_array_index(b, i);
+		if (cmp(&ax, &bx))
+			return 0;
+	}
+
+	return 1;
 }
 
 static void
 compare_attributes(tattribute *clean, tattribute *new, GPtrArray *mods)
 {
-	int changed = 0;
-	compare_ptr_arrays(attribute_values(clean),
-			   attribute_values(new),
-			   carray_ptr_cmp,
-			   (note_function) note_values,
-			   &changed);
-	if (changed) {
+	if (!ordered_array_equal(attribute_values(clean),
+				 attribute_values(new),
+				 carray_ptr_cmp))
+	{
 		LDAPMod *m = attribute2mods(new);
 		m->mod_op |= LDAP_MOD_REPLACE;
 		g_ptr_array_add(mods, m);
@@ -137,8 +147,8 @@ long_array_invert(GArray *array, int i)
 int
 fastcmp(FILE *s, FILE *t, long p, long q, long n)
 {
-	char *b = xalloc(n); /* XXX */
-	char *c = xalloc(n); /* XXX */
+	char *b = xalloc(n);
+	char *c = xalloc(n);
 	int rc = -1;
 	long p_save;
 	long q_save;
@@ -200,27 +210,6 @@ frob_ava(tentry *entry, int mode, char *ad, char *data, int n)
 	return 0;
 }
 
-#if defined(LIBLDAP21)
-#warning compiling for libldap <= 2.1, running with >= 2.2 will result in segfault
-#define safe_str2dn ldap_str2dn
-#elif defined(LIBLDAP22)
-/*
- * the following is exactly equivalent to ldap_str2dn in libldap >= 2.2,
- * but will fail linking on 2.1.  This way we avoid calling the old 2.1
- * version of ldap_str2dn (leading to a segfault when accessing the result).
- */
-static void
-safe_str2dn(char *str, LDAPDN *out, int flags)
-{
-        struct berval bv;
-        bv.bv_val = str;
-        bv.bv_len = strlen(str);
-        ldap_bv2dn_x(&bv, out, flags);
-}
-#else
-#error oops
-#endif
-
 /*
  * Call frob_ava for every ava in DN's (first) RDN.
  * DN must be valid.
@@ -239,7 +228,7 @@ frob_rdn(tentry *entry, char *dn, int mode)
 	int i;
 	int rc = 0;
 
-	safe_str2dn(dn, &olddn, LDAP_DN_FORMAT_LDAPV3);
+	ldap_str2dn(dn, &olddn, LDAP_DN_FORMAT_LDAPV3);
 
 #ifdef LIBLDAP21
 	rdn = (**olddn)[0];
@@ -248,7 +237,7 @@ frob_rdn(tentry *entry, char *dn, int mode)
 #endif
 	for (i = 0; rdn[i]; i++) {
 		LDAPAVA *ava = rdn[i];
-		char *ad = ava->la_attr.bv_val; /* XXX */
+		char *ad = ava->la_attr.bv_val;
 		struct berval *bv = &ava->la_value;
 		if (frob_ava(entry, mode, ad, bv->bv_val, bv->bv_len) == -1) {
 			rc = -1;
@@ -618,7 +607,7 @@ process_deletions(tparser *p,
 }
 
 /*
- * Die compare_streams-Schleife ist das Herz von ldapvi.
+ * The compare_streams loop is the heart of ldapvi.
  *
  * Read two ldapvi data files in streams CLEAN and DATA and compare them.
  *
