@@ -50,7 +50,7 @@ compare(tparser *p, thandler *handler, void *userdata, GArray *offsets,
 		int n;
 
 		if (!cmdline) {
-			fputs("oops: unexpected error in handler\n", stderr);
+			fputs("error: unexpected error in handler\n", stderr);
 			exit(1);
 		}
 
@@ -470,7 +470,7 @@ change_mechanism(bind_options *bo)
 		bo->authmethod = LDAP_AUTH_SASL;
 		puts("Switching to SASL authentication.");
 	}
-	bo->sasl_mech = getline("SASL mechanism", bo->sasl_mech);
+	bo->sasl_mech = ldapvi_getline("SASL mechanism", bo->sasl_mech);
 }
 
 static int
@@ -606,11 +606,11 @@ rebind_sasl(LDAP *ld, bind_options *bind_options, char *dir, int verbose)
 		ld, bind_options->user, bind_options->sasl_mech, NULL,
 		NULL, sasl_mode, ldapvi_sasl_interact, defaults);
 
-	sasl_defaults_free(defaults);
 	if (defaults->fd != -1) {
 		finish_sasl_redirection(defaults);
 		free(defaults->pathname);
 	}
+	sasl_defaults_free(defaults);
 
 	if (rc != LDAP_SUCCESS) {
 		ldap_perror(ld, "ldap_sasl_interactive_bind_s");
@@ -724,6 +724,10 @@ do_connect(char *server, bind_options *bind_options,
 	}
 	if (!profileonlyp)
 		init_sasl_arguments(ld, bind_options);
+	if (bind_options->sasl_secprops)
+		if (ldap_set_option(ld, LDAP_OPT_X_SASL_SECPROPS,
+				    bind_options->sasl_secprops))
+			ldaperr(ld, "ldap_set_option(LDAP_OPT_X_SASL_SECPROPS)");
 	if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &drei))
 		ldaperr(ld, "ldap_set_option(LDAP_OPT_PROTOCOL_VERSION)");
 	if (starttls)
@@ -746,10 +750,10 @@ do_connect(char *server, bind_options *bind_options,
 }
 
 /*
- * fixme: brauchen wir das mit dem user?  dann sollten wir hier noch
- * sasl support vorsehen
+ * fixme: do we need the user parameter?  if so, we should add
+ * SASL support here.
  *
- * ldapvi-Kommandozeile konnte auch nicht schaden
+ * An ldapvi command line option for this wouldn't hurt either.
  */
 static int
 save_ldif(tparser *parser, GArray *offsets, char *clean, char *data,
@@ -763,7 +767,7 @@ save_ldif(tparser *parser, GArray *offsets, char *clean, char *data,
 	if (gethostname(name->str + name->len, 300 - name->len) == -1)
 		syserr();
 	name->len = strlen(name->str);
-	g_string_sprintfa(name, "-%d.ldif", getpid());
+	g_string_append_printf(name, "-%d.ldif", getpid());
 
 	if ( (fd = open(name->str, O_WRONLY | O_CREAT | O_EXCL, 0600)) == -1) {
 		int error = errno;
@@ -1014,12 +1018,10 @@ skip(tparser *p, char *dataname, GArray *offsets, cmdline *cmdline)
 			g_array_index(offsets, long, atoi(key)) = -1;
 		free(key);
 	} else {
-                /* Im Normalfall wollen wir einen Eintrag in data
-                 * ueberspringen.  Wenn aber in data nichts mehr steht,
-                 * sind wir ueber die eigentlichen Aenderungen schon
-                 * hinweg und es handelt sich um eine Loeschung.  In
-                 * diesem Fall muessen wir nur das Offset aus der
-                 * Tabelle entfernen. */
+                /* Normally we want to skip an entry in data.
+                 * But if data is exhausted, the remaining entries
+                 * in the offset table are deletions -- just remove
+                 * the offset. */
                 int n;
                 for (n = 0; n < offsets->len; n++)
                         if (g_array_index(offsets, long, n) >= 0) {
@@ -1047,7 +1049,7 @@ entroid_set_entry(LDAP *ld, tentroid *entroid, tentry *entry)
 
 		{
 			char zero = 0;
-			/* PFUSCH!  die GArrays muessen absolut weg! */
+			/* Hack: temporarily null-terminate the GArray for use as a C string. */
 			g_array_append_val(av, zero);
 			av->len--;
 		}
@@ -1414,12 +1416,16 @@ write_file_header(FILE *s, cmdline *cmdline)
 	int nlines = 0;
 
 	if (print_binary_mode == PRINT_UTF8 && !cmdline->ldif) {
-		fputs("# -*- coding: utf-8 -*- vim:encoding=utf-8:\n", s);
+		/* Emacs coding cookie + vim modeline for UTF-8.
+		 * Note: vim's "encoding" is disallowed in modelines;
+		 * use "fileencoding" instead. */
+		fputs("# -*- coding: utf-8 -*- vim:ft=ldif:fileencoding=utf-8:\n", s);
 		nlines++;
 	}
 	if (cmdline->ldif) {
+		fputs("# -*- coding: utf-8 -*- vim:ft=ldif:fileencoding=utf-8:\n", s);
 		fputs("# " RFC_2849_URL "\n" "# " MANUAL_LDIF_URL "\n", s);
-		nlines += 2;
+		nlines += 3;
 	} else  {
 		fputs("# " MANUAL_SYNTAX_URL "\n", s);
 		nlines++;
@@ -1465,7 +1471,7 @@ copy_sasl_output(FILE *out, char *sasl)
 	int line = 0;
 	int c;
 
-	if (lstat(sasl, &st) == -1) return;
+	if (lstat(sasl, &st) == -1) return 0;
 	if ( !(in = fopen(sasl, "r"))) syserr();
 
 	if (st.st_size > 0) {
@@ -1529,12 +1535,12 @@ main_write_files(
 			if (!source)
 				source = stdin;
 			if (!can_seek(source)) {
-				/* einfach clean als tmpfile nehmen */
+				/* use the clean file as a temporary buffer */
 				if ( !(tmp = fopen(clean, "w+"))) syserr();
 				fcopy(source, tmp);
 				if (fseek(tmp, 0, SEEK_SET) == -1) syserr();
 				source = tmp;
-				/* source war stdin, kann offen bleiben */
+				/* source was stdin, no need to close it */
 			}
 		}
 
