@@ -83,145 +83,12 @@ fcopy(FILE *src, FILE *dst)
 	}
 }
 
-static void
-print_charbag(char *charbag)
-{
-	int i;
-	putchar('[');
-	for (i = 0; charbag[i]; i++) {
-		char c = charbag[i];
-		if (c > 32)
-			putchar(c);
-	}
-	putchar(']');
-}
-
-
-char
-choose(char *prompt, char *charbag, char *help)
-{
-	struct termios term;
-	int c;
-
-	if (tcgetattr(0, &term) == -1) syserr();
-	term.c_lflag &= ~ICANON;
-        term.c_cc[VMIN] = 1;
-        term.c_cc[VTIME] = 0;
-	for (;;) {
-		if (tcsetattr(0, TCSANOW, &term) == -1) syserr();
-		fputs(prompt, stdout);
-		putchar(' ');
-		print_charbag(charbag);
-		putchar(' ');
-		if (strchr(charbag, c = getchar()))
-			break;
-		fputs("\nPlease enter one of ", stdout);
-		print_charbag(charbag);
-		putchar('\n');
-		if (help) printf("  %s", help);
-		putchar('\n');
-	}
-	term.c_lflag |= ICANON;
-	if (tcsetattr(0, TCSANOW, &term) == -1) syserr();
-	putchar('\n');
-	return c;
-}
-
-static long
-line_number(char *pathname, long pos)
-{
-	FILE *f;
-	long line = 1;
-	int c;
-
-	if ( !(f = fopen(pathname, "r+"))) syserr();
-	while (pos > 0) {
-		switch ( c = getc_unlocked(f)) {
-		case EOF:
-			goto done;
-		case '\n':
-			if ( (c = getc_unlocked(f)) != EOF) {
-				ungetc(c, f);
-				line++;
-			}
-			/* fall through */
-		default:
-			pos--;
-		}
-	}
-done:
-	if (fclose(f) == EOF) syserr();
-	return line;
-}
-
-void
-edit(char *pathname, long line)
-{
-	int childpid;
-	int status;
-	char *vi;
-
-	vi = getenv("VISUAL");
-	if (!vi) vi = getenv("EDITOR");
-	if (!vi) vi = "vi";
-
-	switch ( (childpid = fork())) {
-	case -1:
-		syserr();
-	case 0:
-		if (line > 0) {
-			char buf[20];
-			snprintf(buf, 20, "+%ld", line);
-			execlp(vi, vi, buf, pathname, 0);
-		} else
-			execlp(vi, vi, pathname, 0);
-		syserr();
-	}
-
-	if (waitpid(childpid, &status, 0) == -1) syserr();
-	if (!WIFEXITED(status) || WEXITSTATUS(status))
-		yourfault("editor died");
-}
-
-void
-edit_pos(char *pathname, long pos)
-{
-	edit(pathname, pos > 0 ? line_number(pathname, pos) : -1);
-}
-
 static int
 invalidp(char *ti)
 {
 	return ti == 0 || ti == (char *) -1;
 }
 
-void
-view(char *pathname)
-{
-	int childpid;
-	int status;
-	char *pg;
-	char *clear = tigetstr("clear");
-
-	pg = getenv("PAGER");
-	if (!pg) pg = "less";
-
-	if (!invalidp(clear))
-		putp(clear);
-
-	switch ( (childpid = fork())) {
-	case -1:
-		syserr();
-	case 0:
-		execlp(pg, pg, pathname, 0);
-		syserr();
-	}
-
-	if (waitpid(childpid, &status, 0) == -1) syserr();
-	if (!WIFEXITED(status) || WEXITSTATUS(status))
-		puts("pager died");
-}
-
 int
 pipeview(int *fd)
 {
@@ -245,7 +112,7 @@ pipeview(int *fd)
 		close(fds[1]);
 		dup2(fds[0], 0);
 		close(fds[0]);
-		execlp(pg, pg, 0);
+		execl("/bin/sh", "sh", "-c", "exec $0", pg, (char *) NULL);
 		syserr();
 	}
 
@@ -315,7 +182,7 @@ write_ldapvi_history()
 }
 
 char *
-getline(char *prompt, char *value)
+ldapvi_getline(char *prompt, char *value)
 {
 	tdialog d;
 	init_dialog(&d, DIALOG_DEFAULT, prompt, value);
@@ -582,7 +449,7 @@ dialog_rebuild(char *up, char *clreos,
 		putp(up);
 }
 
-static Keymap
+static void
 set_meta_keymap(Keymap keymap, Keymap meta_keymap)
 {
 	if (!meta_keymap)
@@ -640,7 +507,7 @@ dialog(char *header, tdialog *d, int n, int start)
 	}
 
 	rl_variable_bind("horizontal-scroll-mode", "on");
-	rl_inhibit_completion = 1; /* fixme */
+	rl_inhibit_completion = 1; /* completion is not useful in dialogs */
 
 	for (i = 0; i < n; i++)
 		max = MAX(max, strlen(d[i].prompt));
