@@ -15,8 +15,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#define _XOPEN_SOURCE
 #include <unistd.h>
+#include <crypt.h>
 #include "common.h"
 
 #define fast_g_string_append_c(gstring, c)				\
@@ -239,7 +239,7 @@ read_line1(FILE *s, GString *name, GString *value)
 			fputs("Error: Invalid Base64 string.\n", stderr);
 			return -1;
 		}
-		value->len = len;
+		g_string_truncate(value, len);
 	} else if (!strcmp(encoding, "<")) {
 		if (read_ldif_attrval(s, value) == -1) return -1;
 		if (strncmp(value->str, "file://", 7)) {
@@ -443,14 +443,14 @@ error:
 }
 
 /*
- * Lies die erste Zeile eines beliebigen Records nach position `offset' in `s'.
- * Setze *pos (falls pos != 0).
- * Liefere 0 bei Erfolg, -1 sonst.
- * Bei Erfolg:
- *   - pos ist die exakte Anfangsposition.
- *   - Setze *key auf den Schluessel (falls key != 0).
- *   - Setze *dn auf den Distinguished Name (falls dn != 0).
- * EOF ist kein Fehler und liefert *key = 0 (falls key != 0);
+ * Read the first line of any record at position `offset' in `s'.
+ * Set *pos (if pos != 0).
+ * Return 0 on success, -1 otherwise.
+ * On success:
+ *   - pos is the exact starting position.
+ *   - Set *key to the key (if key != 0).
+ *   - Set *dn to the Distinguished Name (if dn != 0).
+ * EOF is not an error and yields *key = 0 (if key != 0).
  */
 static int
 read_header(GString *tmp1, GString *tmp2,
@@ -506,14 +506,14 @@ read_attrval_body(GString *tmp1, GString *tmp2, FILE *s, tentry *entry)
 }
 
 /*
- * Lies ein attrval-record nach position `offset' in `s'.
- * Setze *pos (falls pos != 0).
- * Liefere 0 bei Erfolg, -1 sonst.
- * Bei Erfolg:
- *   - pos ist die exakte Anfangsposition.
- *   - Setze *entry auf den gelesenen Eintrag (falls entry != 0).
- *   - Setze *key auf den Schluessel (falls key != 0).
- * EOF ist kein Fehler und liefert *key = 0 (falls key != 0);
+ * Read an attrval record at position `offset' in `s'.
+ * Set *pos (if pos != 0).
+ * Return 0 on success, -1 otherwise.
+ * On success:
+ *   - pos is the exact starting position.
+ *   - Set *entry to the parsed entry (if entry != 0).
+ *   - Set *key to the key (if key != 0).
+ * EOF is not an error and yields *key = 0 (if key != 0).
  */
 int
 read_entry(FILE *s, long offset, char **key, tentry **entry, long *pos)
@@ -549,12 +549,12 @@ cleanup:
 }
 
 /*
- * Lies die erste Zeile eines beliebigen Records nach position `offset' in `s'.
- * Setze *pos (falls pos != 0).
- * Liefere 0 bei Erfolg, -1 sonst.
- * Bei Erfolg:
- *   - pos ist die exakte Anfangsposition.
- *   - Setze *key auf den Schluessel (falls key != 0).
+ * Read the first line of any record at position `offset' in `s'.
+ * Set *pos (if pos != 0).
+ * Return 0 on success, -1 otherwise.
+ * On success:
+ *   - pos is the exact starting position.
+ *   - Set *key to the key (if key != 0).
  */
 int
 peek_entry(FILE *s, long offset, char **key, long *pos)
@@ -569,12 +569,12 @@ peek_entry(FILE *s, long offset, char **key, long *pos)
 }
 
 /*
- * Lies ein rename-record nach position `offset' in `s'.
- * Liefere 0 bei Erfolg, -1 sonst.
- * Bei Erfolg:
- *   - Setze *dn1 auf den alten DN.
- *   - Setze *dn2 auf den neuen DN.
- *   - *deleteoldrdn auf 1 oder 0;
+ * Read a rename record at position `offset' in `s'.
+ * Return 0 on success, -1 otherwise.
+ * On success:
+ *   - Set *dn1 to the old DN.
+ *   - Set *dn2 to the new DN.
+ *   - Set *deleteoldrdn to 1 or 0.
  */
 int
 read_rename(FILE *s, long offset, char **dn1, char **dn2, int *deleteoldrdn)
@@ -630,11 +630,11 @@ read_delete(FILE *s, long offset, char **dn)
 }
 
 /*
- * Lies ein modify-record nach position `offset' in `s'.
- * Liefere 0 bei Erfolg, -1 sonst.
- * Bei Erfolg:
- *   - Setze *dn auf den DN.
- *   - Setze *mods auf die Aenderungen.
+ * Read a modify record at position `offset' in `s'.
+ * Return 0 on success, -1 otherwise.
+ * On success:
+ *   - Set *dn to the DN.
+ *   - Set *mods to the modifications.
  */
 int
 read_modify(FILE *s, long offset, char **dn, LDAPMod ***mods)
@@ -675,7 +675,7 @@ read_modify(FILE *s, long offset, char **dn, LDAPMod ***mods)
  *   -1 on parse error
  */
 /*
- * FIXME: Warum lesen wir hier nicht einfach bis zur naechsten leeren Zeile?
+ * FIXME: why not just read until the next blank line?
  */
 int
 skip_entry(FILE *s, long offset, char **key)
